#include "io.h"
#include "c.h"

#include <unistd.h>
#include <stdio.h>
#include <errno.h>

typedef ssize_t (*libc_io_func_t)(int fd, void *buf, size_t size);

static int _do_n(int fd, char *buffer, unsigned long size,
                 libc_io_func_t libc_io_func,
                 char *function_name)
{
    char *p = buffer;
    unsigned long bytes_left = size;
    ssize_t successful_bytes;
    
    while(bytes_left) {
        successful_bytes = (*libc_io_func)(fd, p, bytes_left);
        if(-1 == successful_bytes) {
            if(EINTR == errno)
                continue;
            perror(function_name);
            return -1;
        }
        if(0 == successful_bytes) {
            return IO_EOF;
        }
        p += successful_bytes;
        bytes_left -= successful_bytes;
    }
    return 0;
}

int cinvoke_readn(int fd, char *buffer, unsigned long size)
{
    return _do_n(fd, buffer, size, &read, "read");
}

int cinvoke_writen(int fd, const char *buffer, unsigned long size)
{
    return _do_n(fd, (char *)buffer, size, (libc_io_func_t)&write, "write");
}

static char waste_buffer[8*1024];

int cinvoke_wasten(int fd, unsigned long size)
{
    int rc;
    unsigned long bytes_left = size;

    while(bytes_left) {
        unsigned long chunk_size = CINVOKE_MIN(bytes_left, sizeof(waste_buffer));
        rc = cinvoke_readn(fd, waste_buffer, chunk_size);
        if(-1 == rc) {
            return rc;
        }
        bytes_left -= chunk_size;
    }
    return 0;
}
