/* This is the server exports file. All things exposed here are
 * exported. */

#ifndef __cinvoke_server_exports_h_
#define __cinvoke_server_exports_h_

#include "server_type.h"

/* The server functions are only partially reentrant, and none are meant for
 * multi-threaded use.  Use at most one server per thread, and only use within
 * signals if the functions's comment explicitly allows it.
 */

/* The struct fields are intentionally undefined here. They are
 * private, do not use them. */
typedef struct cinvoke_server cinvoke_server_t;

/* Initialize a CInvoke server. in_fd and out_fd may be the same fd,
 * and must support read/write operations.
 *
 * Returns 0 upon successful completion.
 *
 * Must only be called on an uninitialized object.
 */
int cinvoke_server_init(cinvoke_server_t *server, int in_fd, int out_fd);

/* Move the CInvoke server to work with a new set of file descriptors.  The
 * file descriptors should represent the same actual files. This allows
 * migrating the server to alternate file descriptors.
 */
void cinvoke_server_migrate_fds(cinvoke_server_t *server, int in_fd, int out_fd);

/* Finalize a CInvoke server. Further method calls are disallowed (besides
 * init) */
void cinvoke_server_fini(cinvoke_server_t *server);

/* Handle a single request from a client. This function should not be called
 * from within an existing server loop, but only if no loop is running.
 *
 * Returns 0 upon successful completion.
 */
int cinvoke_server_handle_request(cinvoke_server_t *server);

/* Handle requests from a client in a synchronous loop, until an exit command
 * is issued. This function is reentrant, but not signal reentrant - unless
 * guaranteed that the signal doesn't occur during another cinvoke operation.
 * 
 * Returns -1 on error (including EOF)
 *
 * Returns 0 if exit command was issued.
 *    result is set according to the result in the exit command.
 */
int cinvoke_server_loop(cinvoke_server_t *server, long *result,
                        const void *loop_data, unsigned long loop_data_size);

/* The main server loop.
 *
 * This function is not reentrant at all, use the cinvoke_server_loop for
 * nested cinvoke server loops. */
int cinvoke_server_main_loop(cinvoke_server_t *server, long *result);

/* Send an event buffer to the client. This function is fully signal-reentrant,
 * and may be called in signals. */
int cinvoke_server_send_event(cinvoke_server_t *server, const void *buffer, unsigned long buffer_size);

#endif
