from DataTypes import CDataTypes
from DataTypes.SliceUtils import slice_len, normalize_slice
from DataTypes.ItemHandling import ItemHandling
from DataTypes.TypeUtils import classname

import weakref

class AddressSpace(ItemHandling):
    def __init__(self, client):
        self.client = weakref.proxy(client)
        self.offset = 0
        self.address_space = self
        
    def _get_slice(self, index):
        index = normalize_slice(index)
        assert index.step in [None, 1]
        length = slice_len(index)
        if not length:
            return ''
        buf = CDataTypes.buffer_value(self.client.stack, '\x00' * length)
        self.client.functions.memcpy(buf.offset, index.start, length)
        return buf.value
    
    def _set_slice(self, index, value):
        index = normalize_slice(index)
        assert index.step in [None, 1]
        length = slice_len(index)
        if len(value) != length:
            raise ValueError("%s cannot be resized via slice assignment" % (classname(self),))
        if not value:
            return
        buf = CDataTypes.buffer_value(self.client.stack, value)
        self.client.functions_no_response.memcpy(index.start, buf.offset, length)
