from DataTypes.SliceUtils import slice_len, slice_indices
from DataTypes.ItemHandling import ItemHandling
from DataTypes.TypeUtils import classname

class StackArea(ItemHandling):
    def __init__(self, offset, address_space, server_stack, client_stack, get_barrier):
        self.offset = offset
        self.address_space = address_space
        
        assert len(server_stack) == len(client_stack)
        self._length = len(server_stack)
        
        self.server_stack = server_stack
        self.client_stack = client_stack
        self.get_barrier = get_barrier

    def __hash__(self):
        raise TypeError("StackArea is unhashable")

    def __cmp__(self, other):
        return cmp(self[:], other)

    def __len__(self):
        return self._length

    def _fix_slice(self, index):
        return slice_indices(index, len(self))
    
    def _get_slice(self, index):
        index = self._fix_slice(index)
        assert index.step == 1
        barrier = self.get_barrier()
        return (self.server_stack[index.start:min(index.stop, barrier)] +
                self.client_stack[max(index.start, barrier):index.stop])
    
    def _set_slice(self, index, value):
        index = self._fix_slice(index)
        assert index.step == 1
        length = slice_len(index)
        if len(value) != length:
            raise ValueError("%s cannot be resized via slice assignment" % (classname(self),))
        barrier = self.get_barrier()

        start = max(barrier, index.start)
        stop = min(index.stop, barrier)

        server_slice_len = max(0, stop - index.start)
        client_slice_len = max(0, index.stop - start)

        assert server_slice_len + client_slice_len == len(value)

        self.server_stack[index.start:stop] = value[:server_slice_len]
        self.client_stack[start:index.stop] = value[server_slice_len:]
