from DataTypes.Space import attrspace_property, itemspace_property
from NamedValueNode import NamedValueNode
from FormatNodes import Int

# Metaclass, close your eyes.
class EnumType(type):
    def __init__(self, name, bases, namespace):
        super(EnumType, self).__init__(name, bases, namespace)

        self._update_dicts()

    def set_value_names(self, value_names):
        self.value_names = value_names
        self._update_dicts()

    def _update_dicts(self):
        # Still abstract class...
        if self.value_names is None:
            return

        self._value_of_name = dict((name, value)
                                  for name, value in self.value_names)
        self._first_name_of_value = dict((value, name)
                                         for name, value in self.value_names[::-1])
        if len(self._value_of_name) != len(self.value_names):
            raise ValueError("Non-unique names in Enum %r" % (self,))

    def __get_value_of_name(self, name):
        if name not in self._value_of_name:
            raise AttributeError("No such value name", name)
        return self._value_of_name[name]
    names = attrspace_property(__get_value_of_name)

    def __get_first_name_of_value(self, value):
        if value not in self._first_name_of_value:
            raise AttributeError("No such value", value)
        return self._first_name_of_value[value]
    values = itemspace_property(__get_first_name_of_value)
# you can open them now.

class Enum(NamedValueNode):
    __metaclass__ = EnumType

    # C defaults to int, so why shouldn't we? :-)
    node_type = Int

    # Must not change value_names after it is set!
    value_names = None

    def _value_name_of_value(self, value):
        return self._first_name_of_value.get(value, "*Unknown*(%r)" % (value,))
    
    def _value_of_value_name(self, value_name):
        if value_name not in self._value_of_name:
            raise ValueError("%r does not contains the value name" % (self,), value_name)
        return self._value_of_name[value_name]

def enum_on(enum_node_type):
    class TempEnum(Enum):
        node_type = enum_node_type
    TempEnum.__name__ = '%sEnum' % (enum_node_type.__name__,)
    return TempEnum

def enum_of(node_type, enum_value_names, name=None):
    if name is None:
        name = 'Enum_%s_Anonymous' % (node_type.__name__,)
    enum = enum_on(node_type)
    class TempEnum(enum):
        value_names = enum_value_names
    TempEnum.__name__ = name
    return TempEnum
