from DataTypes.SubSequence import SubSequence
from DataTypes.FuncCache import func_cache
from FormatNodes import UnsignedLong
from Errors import DerefError

class Pointer(UnsignedLong):
    to_type = None
    def deref(self):
        if self.to_type is None:
            raise DerefError('Pointer to unknown type cannot be dereferenced')
        address_space = self.environment.get('address_space')
        if address_space is None:
            raise DerefError('Pointer without an address_space cannot be dereferenced')
        value = self.value
        subsequence = SubSequence(address_space, slice(value, value+self.to_type.size()))
        return self.to_type(subsequence, self.environment)

@func_cache
def ptr_type(node_type):
    class TempPointer(Pointer):
        to_type = node_type
    TempPointer.__name__ = 'PointerTo%s' % (node_type.__name__,)
    return TempPointer

def ptr_to(mbuf_type, node, environment=None):
    """Create a pointer that points to the offset of the given node"""
    if environment is None:
        environment=dict(address_space=node.address_space)
    return ptr_type(type(node)).from_value(mbuf_type, node.offset,
                                           environment=environment)
