import array

def array_of(x):
    if isinstance(x, mstr):
        return x._array
    return array.array('c', str(x))

class mstr(object):
    def __init__(self, x=''):
        self._array = array.array('c', str(x))
    def __str__(self):
        return self._array.tostring()
    def __repr__(self):
        return 'mstr(%r)' % (str(self),)

    def __hash__(self):
        raise TypeError("%r objects are mutable and thus unhashable" % (self,))

    def __getitem__(self, index):
        return str(self)[index]
    def __setitem__(self, index, value):
        if isinstance(index, slice):
            self._array[index] = array.array('c', str(value))
        else:
            self._array[index] = value
    def __delitem__(self, index):
        del self._array[index]
    def __add__(self, other):
        return type(self)(str(self)+str(other))
    def __mod__(self, other):
        return type(self)(str(self)%other)
    def __mul__(self, other):
        return type(self)(str(self)*other)
    def __iadd__(self, other):
        self._array += array_of(other)
        return self
    def __imul__(self, other):
        self._array *= other
        return self
    def __imod__(self, other):
        self._array[:] = array.array('c', self % other)
        return self

    def __cmp__(self, other):
        return cmp(str(self), other)
    def __nonzero__(self):
        return bool(self._array)

str_methods = [
    '__len__',
    '__contains__',
    'capitalize', 'center', 'count', 'decode', 'encode', 'endswith', 'expandtabs',
    'find', 'index', 'isalnum', 'isalpha', 'isdigit', 'islower', 'isspace', 'istitle', 'isupper', 'join',
    'ljust', 'lower', 'lstrip', 'replace', 'rfind', 'rindex', 'rjust', 'rsplit', 'rstrip', 'split',
    'splitlines', 'startswith', 'strip', 'swapcase', 'title', 'translate', 'upper', 'zfill'
]
def create_method_proxy(method_name):
    def method(self, *args, **kw):
        return getattr(str(self), method_name)(*args, **kw)
    return method
for x in str_methods:
    method = create_method_proxy(x)
    method.func_name = x
    setattr(mstr, x, method)
