"""Slice utils.

These are functions that should have been slice methods, but arent"""

from SequenceUtils import normalize_index

def single_index_slice(index, length):
    index = normalize_index(index, length)
    return slice(index, index + 1)

def slice_indices(slice_obj, length):
    start, stop, step = slice_obj.indices(length)
    if start > stop:
        start = stop
    return slice(start, stop, step)

def normalize_slice(slice_obj, length=None):
    """Normalize a slice object to a canonical form.

    Converts None's to actual numbers.
    Fixes the slice not to be of a negative length.
    Converts negative indexing to positive indexing, if length is given."""
    start, stop, step = slice_obj.start, slice_obj.stop, slice_obj.step
    if step is None:
        step = 1
    
    if step > 0:
        seqstart, seqstop = 0, length
    else:
        seqstart, seqstop = length, -1
    if start is None:
        start = seqstart
    else:
        start = normalize_index(start, length, check_range=False)
    if stop is None:
        stop = seqstop 
    else:
        stop = normalize_index(stop, length, check_range=False)
    if start is not None and stop is not None and start > stop:
        stop = start
    return slice(start, stop, step)

def fixup_slice(slice_obj, length=None):
    if length is not None:
        return slice_indices(slice_obj, length)
    return normalize_slice(slice_obj)

def combine_slices(a, b):
    for step in [a.step, b.step]:
        assert step in [None, 1], "Only step=1 is currently supported"
    a_start, a_stop = a.start, a.stop
    start, stop = b.start, b.stop
    if a_start is None:
        a_start = 0
    if start is None:
        start = 0

    start += a_start
    if stop is None:
        stop = a_stop
    else:
        stop += a_start
        if a_stop is not None and stop is not None:
            stop = min(stop, a_stop)

    return slice(start, stop, None)

def combine_slice_index(slice_obj, index):
    assert slice_obj.step in [None, 1], "Only step=1 is currently supported"
    if index < 0:
        if slice_obj.stop is None:
            raise IndexError("Negative indices used on a sequence of unknown length")
        index += slice_obj.stop
        if index < 0:
            raise IndexError("Negative index out of bound")
        return index
    index += slice_obj.start
    if index >= slice_obj.stop:
        raise IndexError("Access out of bounds of slice")
    return index

def slice_len(slice_obj, length=None):
    slice_obj = fixup_slice(slice_obj, length)
    start, stop, step = slice_obj.start, slice_obj.stop, slice_obj.step
    assert step in [None, 1], "Only step=1 is currently supported"
    if None in [start, stop]:
        raise TypeError("Undetermined slice length")
    return stop - start
