import keyword
from BoundFunc import BoundFunc

def convert_name(name):
    for kw in keyword.kwlist:
        if len(name) > len(kw) and kw.ljust(len(name), '_') == name:
            return name[:-1]
    return name

def run_func(func, args, err):
    if func is None:
        raise err
    return func(*args)

class Space(object):
    def __init__(self, fget=None, fset=None, fdel=None, doc=None):
        self.__dict__['fget']=fget
        self.__dict__['fset']=fset
        self.__dict__['fdel']=fdel
        self.__dict__['__doc__']=doc

class AttrSpace(Space):
    def __getattr__(self, name):
        return run_func(self.fget, (convert_name(name),),
                        err=AttributeError("Attributes of this object are not readable"))
    def __setattr__(self, name, value):
        return run_func(self.fset, (convert_name(name), value),
                        err=AttributeError("Attributes of this object are not writable"))
    def __delattr__(self, name):
        return run_func(self.fdel, (convert_name(name),),
                        err=AttributeError("Attributes of this object are not deletable"))

class ItemSpace(Space):
    def __getitem__(self, index):
        return run_func(self.fget, (index,),
                        err=TypeError("Items of this object are not readable"))
    def __setitem__(self, index, value):
        return run_func(self.fset, (index, value),
                        err=TypeError("Items of this object are not writable"))
    def __delitem__(self, index):
        return run_func(self.fdel, (index,),
                        err=TypeError("Items of this object are not deletable"))

def space_property(space_type, fget=None, fset=None, fdel=None, doc=None):
    def create_space(self):
        # Must localize these variables to be able to conditionally override them...
        _fget, _fset, _fdel = fget, fset, fdel
        if _fget is not None:
            _fget = BoundFunc(_fget, self)
        if _fset is not None:
            _fset = BoundFunc(_fset, self)
        if _fdel is not None:
            _fdel = BoundFunc(_fdel, self)
        return space_type(_fget, _fset, _fdel)
    return property(create_space, doc=doc)

def attrspace_property(fget=None, fset=None, fdel=None, doc=None):
    return space_property(AttrSpace, fget, fset, fdel, doc)

def itemspace_property(fget=None, fset=None, fdel=None, doc=None):
    return space_property(ItemSpace, fget, fset, fdel, doc)
